from __future__ import absolute_import, division, print_function

# Format expected by setup.py and doc/source/conf.py: string of form "X.Y.Z"
_version_major = 0
_version_minor = 1
_version_micro = 7 # use '' for first of series, number for 1 and above
# _version_extra = 'dev'
_version_extra = ''  # Uncomment this for full releases

# Construct full version string from these.
_ver = [_version_major, _version_minor]
if _version_micro:
    _ver.append(_version_micro)
if _version_extra:
    _ver.append(_version_extra)

__version__ = '.'.join(map(str, _ver))

CLASSIFIERS = ["Development Status :: 3 - Alpha",
               "Environment :: Console",
               "License :: OSI Approved :: BSD License",
               "Operating System :: OS Independent",
               "Programming Language :: Python"]

# Description should be a one-liner:
description = "popylar: record software use with Google Analytics"

# Long description will go up on the pypi page
long_description = """

Popylar
========
Record software use with Google Analytics.

License
=======
``popylar`` is licensed under the terms of the BSD license. See the file
"LICENSE" for information on the history of this software, terms & conditions
for usage, and a DISCLAIMER OF ALL WARRANTIES.

All trademarks referenced herein are property of their respective holders.

Copyright (c) 2016--, Ariel Rokem, The University of Washington
eScience Institute.
"""

NAME = "popylar"
MAINTAINER = "Ariel Rokem"
MAINTAINER_EMAIL = "arokem@gmail.com"
DESCRIPTION = description
LONG_DESCRIPTION = long_description
URL = "http://github.com/popylar/popylar"
DOWNLOAD_URL = ""
LICENSE = "BSD"
AUTHOR = "Ariel Rokem"
AUTHOR_EMAIL = "arokem@gmail.com"
PLATFORMS = "OS Independent"
MAJOR = _version_major
MINOR = _version_minor
MICRO = _version_micro
VERSION = __version__
REQUIRES = ["requests"]



import os.path as op
import requests
import uuid
import configparser

popylar_path = op.join(op.expanduser('~'), '.popylar')


def get_or_create_config():
    if not op.exists(popylar_path):
        parser = configparser.ConfigParser()
        parser.read_dict(dict(user=dict(uid=uuid.uuid1().hex,
                                        track=True)))
        with open(popylar_path, 'w') as fhandle:
            parser.write(fhandle)
    else:
        parser = configparser.ConfigParser()
        parser.read(popylar_path)

    return parser


def opt_out():
    """Permanently opt-out of Popylar tracking.

    To opt-in again, run ``popylar.reset_uid()``
    """
    parser = get_or_create_config()
    parser['user']['track'] = False
    with open(popylar_path, 'w') as fhandle:
        parser.write(fhandle)


def reset_uid():
    """Opt-in to popylar tracking, and/or reset the user id"""
    parser = get_or_create_config()
    parser.read_dict(dict(user=dict(uid=uuid.uuid1().hex,
                                    track=True)))
    with open(popylar_path, 'w') as fhandle:
        parser.write(fhandle)


def opt_in():
    """Opt-in to popylar tracking"""
    if not _get_uid():
        reset_uid()


def _get_uid():
    parser = get_or_create_config()
    if parser['user'].getboolean('track'):
        uid = parser['user']['uid']
    else:
        uid = False
    return uid


def track_event(tracking_id, category, action, uid=None, label=None, value=0,
                software_version=None, timeout=2):
    """
    Record an event with Google Analytics

    Parameters
    ----------
    tracking_id : str
        Google Analytics tracking ID.
    category : str
        Event category.
    action : str
        Event action.
    uid : str
        User unique ID, assigned when popylar was installed.
    label : str
        Event label.
    value : int
        Event value.
    software_version : str
        Records a version of the software.
    timeout : float
        Maximal duration (in seconds) for the network connection to track the
        event. After this duration has elapsed with no response (e.g., on a
        slow network connection), the tracking is dropped.
    """
    # If no user unique ID provided, try to get one from popylar_path:
    if uid is None:
        uid = _get_uid()

    # If it's stil None, assume that the user has opted out
    # (by running popylar.opt_out() or editing the config file)
    if not uid:
        return False

    # Based on https://cloud.google.com/appengine/docs/python/google-analytics
    # and:  https://developers.google.com/analytics/devguides/collection/protocol/v1/devguide  # noqa
    data = {'v': '1',  # API version.
            'tid': tracking_id,  # GA tracking ID
            'cid': uid,  # User unique ID, stored in `popylar_path`
            't': 'event',  # Event hit type.
            'ec': category,  # Event category.
            'ea': action,  # Event action.
            'el': label,  # Event label.
            'ev': value,  # Event value, must be an integer
            # We hijack "app version" to report the package version:
            'av': software_version}
    try:
        response = requests.post('http://www.google-analytics.com/collect',
                                 data=data, timeout=timeout)

        return response

    except:
        return False



import popylar


def test_track_event():
    """
    Test that event tracking goes through
    """
    r = popylar.track_event("UA-86484662-2", 'test', 'test_track_event')
    assert r.status_code == 200, "response is not OK"


def test_track_event_with_version():
    """
    Test recording the software version (in this case, popylar version)
    """
    from popylar.version import VERSION
    r = popylar.track_event("UA-86484662-2", 'test',
                            'test_track_event_with_version',
                            software_version=VERSION)
    assert r.status_code == 200, "response is not OK"



from setuptools import setup, find_packages
import os
import os.path as op
import uuid
import configparser


ver_file = os.path.join('popylar', 'version.py')
with open(ver_file) as f:
    exec(f.read())

popylar_path = op.join(op.expanduser('~'), '.popylar')

# If UID file does not exist, then generate a UID and save to file.
# We don't overwrite an existing one in order to keep UIDs constant
# when the package is upgraded or installed in a venv.
if not os.path.exists(popylar_path):
    parser = configparser.ConfigParser()
    parser.read_dict(dict(user=dict(uid=uuid.uuid1().hex,
                                    track=True)))
    with open(popylar_path, 'w') as fhandle:
        parser.write(fhandle)

PACKAGES = find_packages()

opts = dict(name=NAME,
            maintainer=MAINTAINER,
            maintainer_email=MAINTAINER_EMAIL,
            description=DESCRIPTION,
            long_description=LONG_DESCRIPTION,
            url=URL,
            packages=PACKAGES,
            download_url=DOWNLOAD_URL,
            license=LICENSE,
            classifiers=CLASSIFIERS,
            author=AUTHOR,
            author_email=AUTHOR_EMAIL,
            platforms=PLATFORMS,
            version=VERSION,
            requires=REQUIRES)


if __name__ == '__main__':
    setup(**opts)
